/* globals lp,phoneValidators */
// version 5.2
;(function(){

  //The reason for the two versions is because we are currently using window.module
  //which will conflict with some thirdparty libraries.  We cannot modify all
  //the pages because many users have created custom scripts refering to
  //window.module;  Instead we will keep users on window.module unless they
  //report problems, when they do we will switch their account to use
  //window.ubModule.
  var ubModule = window.ubModule ? window.ubModule : (window.module || {});

  ubModule.lp = ubModule.lp || {};
  ubModule.lp.form = ubModule.lp.form || {};
  ubModule.lp.form.data = null;

  window.lp = window.lp || {};
  window.lp.form = window.lp.form || {};
  window.lp.form.main = window.lp.form.main || {};

  lp.jQuery(document).ready(function(){ lp.form.main.start(); });

  lp.form.main = (function(){
    var formContainer,
        formButton,
        errorContainer,
        errorLabelContainer,
        formSelector,
        ubafsFieldSelector,
        submitHeaders = {};

    var start = function(){
      if (ubModule.lp.form.data) {
        setupSelectors();
        initialize();
        handleForm();
      }
    };

    var setupSelectors = function() {
      formSelectors();
      errorSelectors();
    };

    var formSelectors = function() {
      formContainer      = '#' + ubModule.lp.form.data.formContainerId;
      formButton         = '#' + ubModule.lp.form.data.formButtonId;
      formSelector       = formContainer + ' form';
      ubafsFieldSelector = formSelector + ' :input[name^="ubafs-"]';
    };

    var errorSelectors = function() {
      errorContainer      = '#'+ubModule.lp.form.data.errorContainerId;
      errorLabelContainer = errorContainer + ' ul';
    };

    var handleForm = function() {
      copyURLParamsToFields();
      formSubmitHandler();
      adjustSelectBoxWidthForIE8();
    };

    var formSubmitHandler = function() {
      lp.jQuery(formSelector).keypress(function(e) {
        if(e.which === 13 && e.target.nodeName.toLowerCase() !== 'textarea') {
          e.preventDefault();
          lp.jQuery(formSelector).submit();
        }
      });
    };

    var positionErrors = function() {
      var errorWidth  = lp.jQuery(errorContainer).outerWidth(true)
        , formWidth   = lp.jQuery(formContainer).outerWidth(true)
        , totalWidth  = formWidth + errorWidth
        , windowWidth = Math.min(lp.jQuery(window).width(), window.innerWidth);
      if(totalWidth < windowWidth) {
        positionErrorsOnMonitors(formWidth);
      } else {
        positionErrorsSmallScreens();
      }
    };

    var initErrorHide = function() {
      lp.jQuery(errorContainer).click(function() {
        lp.jQuery(errorContainer + ' .content ul ').toggleClass('hide');
        updateErrorToggle();
      });
    };

    var updateErrorToggle = function() {
      var $error = lp.jQuery(errorContainer);
      if(lp.jQuery(errorContainer + ' .content ul').is(':hidden')) {
        $error.find('.error-toggle').text('Show');
        $error.find('.error-toggle-arrow').css('background-position', '0px -8px'); // Right-pointing triangle
        $error.find('ul').addClass('hide');
      }
      else {
        $error.find('.error-toggle').text('Hide');
        $error.find('.error-toggle-arrow').css('background-position', '0px 1px'); // Down-pointing pointing triangle
        $error.find('ul').removeClass('hide');
      }
    };

    var positionErrorsOnMonitors = function(formWidth) {
      var formOffset         = lp.jQuery(formContainer).offset()
        , rightEdgePosition  = formOffset.left + formWidth + 16
        , docWidth           = Math.min(lp.jQuery(document).width(), window.innerWidth)
        , rightEdge          = docWidth - rightEdgePosition
        , errorWidthMargin   = 280 + 16
        , shouldPositionLeft = rightEdge < 280;

      var left = shouldPositionLeft ? formOffset.left - errorWidthMargin : rightEdgePosition;
      if (left > 0) {
        var container = lp.jQuery(errorContainer);
        container.css({
          left: left + "px",
          top: Math.max(formOffset.top, lp.jQuery(document).scrollTop()+4) + "px",
          width: "280px"
        });
        window.onscroll = function() {
          handlePositioning(formOffset, container);
        };
      } else {
        // tablet!
        positionErrorsSmallScreens();
      }
    };

    var handlePositioning = function(formPosition, container) {
      var errorElTop = container.offset().top;
      var formElBottom = Math.round(errorElTop + container.innerHeight());
      var formHeight = lp.jQuery(formContainer).innerHeight();
      var formBottom = formPosition.top + formHeight;

      if(formBottom < formElBottom && formPosition.top < lp.jQuery(document).scrollTop()) {
        container.css({position: 'absolute'});
      } else if(formPosition.top < lp.jQuery(document).scrollTop()) {
        container.css({position: 'absolute', top: lp.jQuery(document).scrollTop()});
      } else {
        container.css({position: 'absolute', top: formPosition.top});
      }
    };

    //Small screen error positioning is so that the error will display properly on mobile
    //devices.
    var positionErrorsSmallScreens = function() {
      lp.jQuery(errorContainer).css({
        position: "fixed",
        left: "0px",
        right: "0px",
        top: "0px"
      });
    };

    var getParamsForSuccessModal = function(urlParams) {
      return urlParams ? urlParams.replace(/\?/g, '') : '';
    };

    var getOperatorToUseFromUrl = function(url) {
      return /\?/.test(url) ? '&' : '?';
    };

    var is404 = function() {
      if (window.ub && window.ub.page) {
        var paths = window.ub.page.url.replace(/\/$/, '').split('/').slice(3);
        return paths.length === 1 && paths[0] === '404';
      } else {
        return false;
      }
    };

    var isPreview = function() {
      return location.hostname.indexOf('unbouncepreview.com') > -1;
    };

    var pageIsLightbox = function() {
      // The try/catch is necessary to avoid an exception being thrown if the parent is on
      // a different domain - e.g. when in preview.
      try {
        return window.ub && window.ub.page && window.ub.page.usedAs === 'lightbox' &&
          window.parent !== window && window.parent.lp && window.parent.lp.jQuery;
      } catch(err) {
        return false;
      }
    };

    var targetIsParent = function() {
      return pageIsLightbox() || ubModule.lp.form.data.confirmTarget === '_parent';
    };

    var pageIsEmbeddable = function() {
      // The file that defines window.lp.embeddable (embeddable.js) is only inserted onto embeddable
      // pages, so if that is present we are guaranteed to be on an embeddable.
      return typeof window.lp.embeddable === 'object';
    };

    var buildSuccessModalUrl = function(data, form) {
      var pathPrefix     = is404() && ! isPreview() ? '/404/' : '';

      var modalUrlParts  = data.url.split('?');
      var modalPath      = pathPrefix + modalUrlParts[0];
      var modalParams    = modalUrlParts[1];
      var thisPageParams = getParamsForSuccessModal(window.location.search);
      var formParams     = serializeFormData(form);

      // Re-order the URL parameters such that params included in the form confirmation
      // dialog URL come *after* the params lifted from the current page's URL. This is
      // necessary for preview to work.
      var params = lp.jQuery.grep(
        [thisPageParams, formParams, modalParams],
        function(str) { return str; } // removes empty param strings
      ).join('&');

      return modalPath + (params ? '?' : '') + params;
    };

    var isMobileAllowed = function(data) {
      var mediaQuery = 'screen and (max-width: 600px)';
      var mainPage = pageIsLightbox() ? window.parent : window;
      return (typeof mainPage.matchMedia === "function") &&
        mainPage.matchMedia(mediaQuery).matches &&
        data.size.mobile;
    };

    var getModalSize = function(data) {
      var version = isMobileAllowed(data) ? 'mobile' : 'desktop';

      if (data.size[version]) {
        return {
          width  : data.size[version].width,
          height : data.size[version].height
        };
      }

      return {
        width  : data.size.width,
        height : data.size.height
      };
    };

    var isUsingIosDevice = /iPhone|iPad|iPod/i.test(navigator.userAgent);

    var showSuccessModal = function(data, form) {
      var path = buildSuccessModalUrl(data, form);
      var size = getModalSize(data);
      var targetWindow = pageIsLightbox() ? window.parent : window;

      if (pageIsEmbeddable() && window.lp.ubBrowser.isPostMessageSupported()) {
        window.lp.embeddable.replaceOverlayContent(window.ub.page.url + path, size);
        return;
      }

      this.overlay = targetWindow.lp.jQuery.ubpoverlay({
        href: path,
        padding: 0,
        type: 'iframe',
        onOverlayClick: false,
        width: size.width,
        height: size.height,
        onStart: function() {
          // We're opening a form confirmation dialog instead of a new-style lightbox, so
          // remove the class that adds extra styling.
          targetWindow.lp.jQuery('body').removeClass('ub-lightbox-open');
        },
        onComplete: function() {
          targetWindow.lp.ubBrowser.adjustOverlayWidth(targetWindow);

          // Set focus to the iframe so that the form cannot get resubmitted by mistake.
          // This is related to issue LP-6580.
          targetWindow.lp.jQuery('#ubpoverlay-frame').focus();
        },
        onClosed: function() {
          // This is done to fix 'focus jumping' on iOS devices.
          if (isUsingIosDevice) {
            targetWindow.lp.jQuery('#ios-meta').remove();

            if (targetWindow.ub && targetWindow.ub.page && targetWindow.ub.page.storedScrollPosition)  {
              targetWindow.lp.jQuery('body').scrollTop(targetWindow.ub.page.storedScrollPosition);
              delete targetWindow.ub.page.storedScrollPosition;
            }
          }
        }
      });

      var getAdjustedViewportHeight = function(height) {
        // returns viewport [width, height, left, top]
        var viewHeight = targetWindow.lp.jQuery.ubpoverlay.getViewport()[1];
        return height > viewHeight ? viewHeight : height;
      };

      targetWindow.lp.jQuery(window).resize(function(){
        if ( ! targetWindow.lp.jQuery('body').hasClass('ub-lightbox-open')) {
          // Presence of the ub-lightbox-open class means we're using the new lp.lightbox
          // ubpoverlay code, which doesn't require manual adjustments, unlike the code
          // still being used by the form confirmation dialog.
          var size = getModalSize(data);
          targetWindow.lp.jQuery('#ubpoverlay-wrap, #ubpoverlay-outer, #ubpoverlay-content').css({
            width  : size.width,
            height : getAdjustedViewportHeight(size.height)
          });

          targetWindow.lp.jQuery.ubpoverlay.resize();
        }
      });
    };

    var serializeFormData = function(form) {
      var array = lp.jQuery(form).serializeArray();
      var filteredArray = lp.jQuery.map(array, function(param) {
        return ('pageId' === param.name ||
                'pageVariant' === param.name ||
                /^ubafs-\w+/.test(param.name)
               ) ? null : param;
      });
      return lp.jQuery.param(filteredArray);
    };

    var getSuccessURL = function(url, form) {
      if ( ubModule.lp.form.data.passParams ) {
        return url + getOperatorToUseFromUrl(url) + serializeFormData(form);
      } else {
        return url;
      }
    };

    var navigateToURL = function(url) {
      var targetWindow = targetIsParent() ? window.parent : window;
      targetWindow.location.href = url;
    };

    var enableForm = function() {
      lp.jQuery(formButton).removeClass('disabled');
    };

    var disableForm = function() {
      lp.jQuery(formButton).addClass('disabled');
    };

    var isFormDisabled = function() {
      return lp.jQuery(formButton).hasClass('disabled');
    };

    var getFormAction = function(form) {
      var action = form.getAttribute('action');
      if (typeof action === 'object' && action.nodeType === 1) {
        var parent = action.parentNode;
        var node = parent.removeChild(action);
        action = getFormAction(form);
        parent.appendChild(node);
      }
      return action;
    };

    var setFormAction = function(form, url) {
      var action = form.getAttribute('action');
      if (typeof action === 'object' && action.nodeType === 1) {
        var parent = action.parentNode;
        var node = parent.removeChild(action);
        action = setFormAction(form, url);
        parent.appendChild(node);
      }

      if (ubModule.lp.form.data.passParams) {
        url = url + getOperatorToUseFromUrl(url) + serializeFormData(form);
      }

      form.setAttribute('action', url);

      if (targetIsParent()) {
        form.setAttribute('target', '_parent');
      }
    };

    var stripEmailField = function(validationRules) {
      lp.jQuery.each(validationRules, function(key, value){
        if(value.email) {
          validationRules[key] = {
            required: {
              depends: function(){
                if(/\W$/.test(lp.jQuery(this).val())) {
                  lp.jQuery(this).val(lp.jQuery.trim(lp.jQuery(this).val()));
                }
                return value.required;
              }
            },
            email: true
          };
        }
      });
      return validationRules;
    };

    var phoneValidator = function(type) {
      lp.jQuery.validator.addMethod('phone', function(value, element) {
        value = value.replace(/([. \-()+])/g, '');
        if(type === 'uk') {
          return this.optional(element) || value.length > 9 &&
            value.match(/^((44?(0)?)|(0))([0-9]{3,4}){3}$/);
        } else if(type === 'australian') {
          return this.optional(element) || (value.length  > 5 && value.length < 16) &&
            value.match(/^((0|61)(2|4|3|7|8)){0,1}[0-9]{2}[0-9]{2}[0-9]{1}[0-9]{3}$/);
        } else if(type === 'generic') {
          return this.optional(element) || (value.length > 5 && value.length < 16) && !isNaN(value);
        } else {
          //North American
          return this.optional(element) || (value.length > 9) && value.match(/^(1-?)?(\([2-9]\d{2}\)|[2-9]\d{2})-?[2-9]\d{2}-?\d{4}$/);
        }
        // BUG phoneValidators is not defined anywhere in this project!
        return phoneValidators[type](value, element, this.optional);
      }, 'Please specify a valid phone number');
    };

    var setValidateRules = function(validationData) {
      var validationType = validationData.validationType;
      var validationRules = ['generic', 'australian', 'uk', 'north-american'];
      if((validationType && lp.jQuery.inArray(validationType, validationRules) > -1)) {
        phoneValidator(validationType);
      }
    };

    var notFormPostOrRedirect = function() {
      var formData = ubModule.lp.form.data;
      return lp.jQuery.inArray(formData.confirmAction, ['url', 'post']) === -1 ||
        formData.lightbox === true;
    };

    var highlightErrors = function(errorCount) {
      if ( errorCount === 0 ) {
        lp.jQuery(errorContainer).hide();
      } else {
        var plural = errorCount > 1 ? ' errors' : ' error';
        var msg = 'Please fix the following ' + errorCount + plural + ':';
        lp.jQuery(errorContainer + ' .content .error .error-msg').text(msg);

        updateErrorToggle();
      }
    };

    var initialize = function() {
      positionErrors();
      initErrorHide();
      setValidateRules(ubModule.lp.form.data);

      lp.jQuery(formSelector).validate({
        rules: stripEmailField(ubModule.lp.form.data.validationRules),
        messages: ubModule.lp.form.data.validationMessages,
        errorContainer: errorContainer,
        errorLabelContainer: errorLabelContainer,
        wrapper: 'li',
        unhighlight: function(element, errorClass, validClass) {
          // Gets called when a field changes from invalid to valid. Hides the entire
          // error container if it contains no errors
          var errorCount = Object.keys(this.invalid).length;
          window.setTimeout(highlightErrors.bind(this, errorCount), 5);

          // Retain the default functionality of `unhighlight`:
          lp.jQuery(element).removeClass(errorClass).addClass(validClass);
        },
        highlight: function(element, errorClass, validClass) {
          // Gets called when a field changes from valid to invalid. Hides the entire
          // error container if it contains no errors
          var errorCount = Object.keys(this.invalid).length;
          window.setTimeout(highlightErrors.bind(this, errorCount), 5);


          // Retain the default functionality of `highlight`:
          lp.jQuery(element).addClass(errorClass).removeClass(validClass);
        },
        invalidHandler: positionErrors,
        focusInvalid: false,
        submitHandler: function(form) {
          if (isFormDisabled()) {
            return;
          }

          if (pageIsEmbeddable()) {
            window.lp.embeddable.reportFormSubmit(
              ubModule.lp.form.data.isConversionGoal === true,
              ubModule.lp.form.data.confirmAction
            );
          }

          disableForm();

          lp.jQuery.ajax({
            url: getFormAction(lp.jQuery(form).get(0))+'&lp-form-submit-method=ajax',
            type: 'POST',
            data: lp.jQuery(form).serialize(),
            beforeSend: function(xhr) {
              // Blur input focus to hide mobile keyboards if opened on form submission
              lp.jQuery(formSelector + " .text").blur();
              // Add request headers stored in the outer scope submitHeaders variable
              // (added through public addSubmitHeaders method). Only allow headers that
              // start with X-Ub- as a security precaution.
              if (xhr && typeof xhr.setRequestHeader === 'function') {
                lp.jQuery.each(submitHeaders, function(key, value) {
                  if (/^X-Ub-/.test(key) && typeof value === 'string') {
                    xhr.setRequestHeader(key, value);
                  }
                });
              }
            },
            debug: true,
            error: function() {
              alert(ubModule.lp.form.data.errorMessage ||
                    'We\'re sorry the form could not be submitted because something went wrong. Please try again.');
            },
            success: function(data) {
              if (data.protected_assets) {
                ubModule.lp.form.responseData = {
                  protectedAssets: data.protected_assets
                };
              }
              var $form = lp.jQuery(formSelector);
              var url;

              switch (ubModule.lp.form.data.confirmAction) {
                case 'url':
                  url = getSuccessURL(ubModule.lp.form.data.confirmData, form)
                    .replace(/\+/g, '%20');
                  navigateToURL(url);
                break;

                case 'externalLightbox':
                  url = getSuccessURL(ubModule.lp.form.data.confirmData, form)
                    .replace(/\+/g, '%20');
                  if (window.lp.lightbox && ubModule.lp.form.data.lightbox &&
                    ubModule.lp.form.data.lightboxSize) {
                      window.lp.lightbox.openLightbox(url, ubModule.lp.form.data.lightboxSize);
                  } else {
                    navigateToURL(url);
                  }
                break;

                case 'message':
                  alert(ubModule.lp.form.data.confirmData);
                break;

                case 'modal':
                  showSuccessModal(ubModule.lp.form.data.confirmData, form);
                break;

                case 'post':
                  $form.unbind();
                  setFormAction($form.get(0), ubModule.lp.form.data.confirmData);
                  lp.jQuery(ubafsFieldSelector).remove();
                  $form.submit();
                break;
              }
            },
            complete: function() {
              if (notFormPostOrRedirect()) {
                enableForm();
                form.blur();
                form.reset();
              }
            }
          });
        }
      });
    };

    var copyURLParamsToFields = function() {
      var params = window.lp.form.main.getUrlParams();
      var field;
      var excludeFields = ['pageid', 'pagevariant'];
      for (var param in params) {
        field = lp.jQuery(formSelector).find('input[name='+param+']')[0] || lp.jQuery(formSelector).find('textarea[name='+param+']')[0];
        var isExcludedField = lp.jQuery.inArray( excludeFields, param.toLowerCase() );
        if (typeof field !== 'undefined' && isExcludedField === -1 ) {
          if (field.type === 'text' || field.type === 'hidden' || field.type === 'textarea') {
            //If the parameter values are an array than we make sure that we set the field value to a
            //concatenated string of the values.
            var fieldValue = (params[param] instanceof Array) ? params[param].join(',') : params[param];
            field.value = decodeURLParam(fieldValue);
          }
        }
      }
    };

    var decodeURLParam = function(param) {
      try {
        if (lp.jQuery.isArray(param)) {
          // If there are multiple values for this param, use the last one.
          return decodeURIComponent(param[param.length - 1]);
        } else {
          return decodeURIComponent(param);
        }
      } catch(e) {
        // incoming URL parameters might be ASCII encoded. Force them to UTF-8.
        /* global unescape */
        return decodeURIComponent(encodeURIComponent(unescape(param)));
      }
    };

    //Handle ie8 select box not large enough issue as noted by lp-2467
    var adjustSelectBoxWidthForIE8 = function() {
      if(lp.jQuery.browser.msie && parseInt(lp.jQuery.browser.version,10) < 9) {
        var el;
        lp.jQuery('select').each(function(){
          el = lp.jQuery(this);
          el.data('origWidth', el.outerWidth());
        }).mousedown(function(){
          lp.jQuery(this).css('width', 'auto');
        }).bind("blur change", function(){
          el = lp.jQuery(this);
          el.css('width', el.data('origWidth'));
        });
      }
    };

    var getWindowLocation = function(){
      return window.location.href;
    };

    var getUrlParams = function(){
      var params = {};
      var loc = this.getWindowLocation();
      var hashes = loc.slice(loc.indexOf('?') + 1).split('&');

      var i;
      for (i = 0; i < hashes.length; i++) {
        var hash_data = hashes[i].split('='),
            key       = hash_data[0],
            value     = hash_data[1] || '';

        if (!params.hasOwnProperty(key)) {
          params[key] = value;
        } else if (lp.jQuery.isArray(params[key])) {
          params[key].push(value);
        } else {
          params[key] = [params[key], value];
        }
      }
      return params;
    };

    var addSubmitHeaders = function(headers) {
      // submitHeaders is declared in the outer scope and accessed in the form submitHandler.
      submitHeaders = lp.jQuery.extend({}, submitHeaders, headers);
    };

    return {
      getOperatorToUseFromUrl  : getOperatorToUseFromUrl,
      getParamsForSuccessModal : getParamsForSuccessModal,
      serializeFormData        : serializeFormData,
      buildSuccessModalUrl     : buildSuccessModalUrl,
      getWindowLocation        : getWindowLocation,
      getUrlParams             : getUrlParams,
      getModalSize             : getModalSize,
      start                    : start,
      addSubmitHeaders         : addSubmitHeaders
    };
  })();
})();
